// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_pm_m_h)
#define octave_mx_pm_m_h 1
#include "octave-config.h"
#include "dMatrix.h"
#include "PermMatrix.h"
  extern OCTAVE_API Matrix operator * (const PermMatrix&, const Matrix&);
#endif
