// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_cm_s_h)
#define octave_mx_cm_s_h 1
#include "octave-config.h"
#include "CMatrix.h"
  extern OCTAVE_API ComplexMatrix operator + (const ComplexMatrix&, const double&);
  extern OCTAVE_API ComplexMatrix operator - (const ComplexMatrix&, const double&);
  extern OCTAVE_API ComplexMatrix operator * (const ComplexMatrix&, const double&);
  extern OCTAVE_API ComplexMatrix operator / (const ComplexMatrix&, const double&);
  extern OCTAVE_API boolMatrix mx_el_lt (const ComplexMatrix&, const double&);
  extern OCTAVE_API boolMatrix mx_el_le (const ComplexMatrix&, const double&);
  extern OCTAVE_API boolMatrix mx_el_ge (const ComplexMatrix&, const double&);
  extern OCTAVE_API boolMatrix mx_el_gt (const ComplexMatrix&, const double&);
  extern OCTAVE_API boolMatrix mx_el_eq (const ComplexMatrix&, const double&);
  extern OCTAVE_API boolMatrix mx_el_ne (const ComplexMatrix&, const double&);
  extern OCTAVE_API boolMatrix mx_el_and (const ComplexMatrix&, const double&);
  extern OCTAVE_API boolMatrix mx_el_or (const ComplexMatrix&, const double&);
#endif
